<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * WordPress XML-RPC API用基底クラス
 * 
 * WordPress XML-RPC APIにて共通的に使用する機能を集めた用基底クラス
 * 
 * PHP version 5
 * 
 * LICENSE: This LICENSE is in the New BSD license style.
 *
 * @category  WordPress
 * @package   WordPress_Core
 * @author    Takahiro Ooishi <taka0125@users.sourceforge.jp>
 * @version   SVN : $Id: API.class.php 6 2009-10-03 18:41:22Z taka0125 $
 * @copyright 2009 Takahiro Ooishi
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 * @see       http://codex.wordpress.org/XML-RPC_wp
 */

/**
 * WordPress XML-RPC API用基底クラス
 * 
 * WordPress XML-RPC APIにて共通的に使用する機能を集めた用基底クラス
 * 
 * @category  WordPress
 * @package   WordPress_Core
 * @abstract
 * @author    Takahiro Ooishi <taka0125@users.sourceforge.jp>
 * @version   Release: @package_version@
 * @copyright 2009 Takahiro Ooishi
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 * @see       http://codex.wordpress.org/XML-RPC_wp
 *
 */
abstract class WordPress_Core_API
{
    protected $config;
    protected $errorFlg;
    protected $faultCode;
    protected $faultString;
    protected $connection;

    const DefaultTimeOut = 10;

   /**
    * コンストラクタ
    *
    * @param object $config WordPress_Configクラスのインスタンス
    * @return void
    */
    function __construct($config)
    {
        if (!$this->isValidConfig($config)) throw new Exception('invalid config');
        $this->setConfig($config);
        $this->connection = $this->open();
        $this->setTimeout(self::DefaultTimeOut);
    }

   /**
    * デストラクタ
    *
    * @return void
    */
    function __destruct()
    {
        if (!is_null($this->connection)) {
            $this->close($this->connection);
            $this->connection = null;
        }
    }

   /**
    * configを返す
    *
    * @return object 保持しているconfigを返す
    */
    function config()
    {
        return $this->config;
    }

   /**
    * configを設定する
    *
    * @param object $config WordPress_Configクラスのインスタンス
    * @return void
    */
    function setConfig($config)
    {
        $this->config = $config;
    }

   /**
    * Curlのタイムアウトを設定する
    *
    * @param integer $timeout タイムアウト秒数(1以上)
    * @return bool 成功時:true,失敗時:false
    */
    function setTimeout($timeout)
    {
        if (!is_numeric($timeout)) return false;
        $timeout = intval($timeout);
        if ($timeout < 1) return false;
        
        curl_setopt($this->connection, CURLOPT_TIMEOUT, $timeout);
        return true;
    }

   /**
    * 直近のリクエストの成功／失敗を返す
    *
    * @return bool エラー時:true,成功時:false
    */
    function isError()
    {
        return $this->errorFlg;
    }

   /**
    * 直近のリクエストでのエラー内容を返す
    *
    * @return string エラー内容
    */
    function getError()
    {
        if (!$this->errorFlg) return '';
        return "faultCode={$this->faultCode},faultString={$this->faultString}";
    }

   /**
    * 直近のリクエストでのfaultCodeを返す
    *
    * @return string faultCode
    */
    function faultCode()
    {
        if (!$this->errorFlg) return '';
        return $this->faultCode;
    }

   /**
    * 直近のリクエストでのfaultStringを返す
    *
    * @return string faultString
    */
    function faultString()
    {
        if (!$this->errorFlg) return '';
        return $this->faultString;
    }

   /**
    * エラー情報を初期化する
    *
    * @return void
    */
    protected function clearError()
    {
        $this->errorFlg = false;
        $this->faultCode = null; 
        $this->faultString = null;
    }

   /**
    * リクエストのエラー情報を設定する
    *
    * @param array $results リクエスト結果
    * @return void
    */
    protected function setError($results)
    {
        if ($this->config->outputEncode() !== WordPress_Encode::Utf8) {
            $results = WordPress_Encode::encode($results, $this->config->outputEncode(), WordPress_Encode::Utf8);
        }

        $this->errorFlg = true;
        $this->faultCode = $results['faultCode'];
        $this->faultString = $results['faultString'];
    }

   /**
    * Curlコネクションの作成
    *
    * @return mixed Curlコネクション
    */
    protected function open()
    {
        $header = array();
        $header[] = 'Content-type: text/xml';
        
        $connection = curl_init();
        
        $url = $this->config->xmlrpcUrl();
        
        curl_setopt($connection, CURLOPT_URL, $url);
        curl_setopt($connection, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($connection, CURLOPT_TIMEOUT, 1);
        curl_setopt($connection, CURLOPT_HTTPHEADER, $header);
        
        return $connection;
    }

   /**
    * Curlコネクションの削除
    *
    * @param mixed $connection Curlコネクション
    * @return void
    */
    protected function close($connection)
    {
        curl_close($connection);
    }

   /**
    * XML-RPCリクエスト用XMLの作成
    *
    * @param string $function_name コールする関数名
    * @param array $params パラメータ
    * @return string XML-RPCリクエスト用XML
    */
    protected function buildRequest($function_name, $params)
    {
        $output_options = array(
            'escaping' => array(''),
            'encoding' => 'UTF-8'
        );
        return xmlrpc_encode_request($function_name, $params, $output_options);
    }

   /**
    * XML-RPCリクエスト実行
    *
    * @param string $request XML-RPCリクエスト用XML
    * @return array XML-RPCリクエスト結果
    */
    protected function request($request)
    {
        $this->clearError();

        if (is_null($this->connection)) {
            $this->connection = $this->open();
        }

        curl_setopt($this->connection, CURLOPT_POSTFIELDS, $request);

        $xml = curl_exec($this->connection);
        if (curl_errno($this->connection)) throw new Exception(curl_error($this->connection));

        $results = xmlrpc_decode($xml);
        if (!is_array($results)) return $results;

        if (xmlrpc_is_fault($results)) {
            $this->setError($results);
            return false;
        }

        if ($this->config->outputEncode() !== WordPress_Encode::Utf8) {
            $results = WordPress_Encode::encode($results, $this->config->outputEncode(), WordPress_Encode::Utf8);
        }

        return $results;
    }

   /**
    * 正当なコンフィグオブジェクトかチェック
    *
    * @param object $config チェックするオブジェクト
    * @return bool 正当:true,不正:false
    */
    protected function isValidConfig($config)
    {
        if (! $config instanceof WordPress_Config) return false;
        return true;
    }
}
