<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * WordPress XML-RPC コンフィグ用基底クラス
 *
 * コンフィグ用基底クラス
 *
 * PHP version 5
 *
 * LICENSE: This LICENSE is in the New BSD license style.
 *
 * @category  WordPress
 * @package   WordPress_Core
 * @author    Takahiro Ooishi <taka0125@users.sourceforge.jp>
 * @version   SVN : $Id: Config.class.php 6 2009-10-03 18:41:22Z taka0125 $
 * @copyright 2009 Takahiro Ooishi
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 * @see       http://codex.wordpress.org/XML-RPC_wp
 */

/**
 * WordPress コンフィグ用基底クラス
 *
 * コンフィグ用基底クラス
 *
 * @category  WordPress
 * @package   WordPress_Core
 * @abstract
 * @author    Takahiro Ooishi <taka0125@users.sourceforge.jp>
 * @version   Release: @package_version@
 * @copyright 2009 Takahiro Ooishi
 * @license   http://opensource.org/licenses/bsd-license.php New BSD License
 * @see       http://codex.wordpress.org/XML-RPC_wp
 *
 */
abstract class WordPress_Core_Config
{
    protected $blogUrl;
    protected $xmlrpcUrl;
    protected $outputEncode = null;
    
    protected $username;
    protected $password;
    
    protected $blogId;

   /**
    * コンストラクタ
    *
    * @param string $blog_url ブログURL
    * @param string $username 投稿アカウントのユーザ名
    * @param string $username 投稿アカウントのパスワード
    * @param string $blog_id ブログID
    * @param string $output_encode リクエスト結果の文字コード
    * @return void
    */
    function __construct($blog_url, $username = null, $password = null, $blog_id = null, $output_encode = WordPress_Encode::Utf8)
    {
        if (!$this->isValidBlogUrl($blog_url)) throw new Exception('invalid blog_url');
        $blog_url = $this->normalizeBlogUrl($blog_url);
        $this->blogUrl = $blog_url;
        $this->xmlrpcUrl = $this->buildXmlrpcUrl($blog_url);
        $this->setOutputEncode($output_encode);
        
        $this->setUsername($username);
        $this->setPassword($password);
        
        $this->setBlogId($blog_id);
    }
    
   /**
    * ブログURLを返す
    *
    * @return string ブログURL
    */
    function blogUrl()
    {
        return $this->blogUrl;
    }

   /**
    * XML-RPC用URLを返す
    *
    * @return string XML-RPC用URL
    */
    function xmlrpcUrl()
    {
        return $this->xmlrpcUrl;
    }

   /**
    * リクエスト結果の文字コードを返す
    *
    * @return string リクエスト結果の文字コード
    */
    function outputEncode()
    {
        return $this->outputEncode;
    }

   /**
    * 投稿アカウントのユーザ名
    *
    * @return string 投稿アカウントのユーザ名
    */
    function username()
    {
        return $this->username;
    }

   /**
    * 投稿アカウントのパスワード
    *
    * @return string 投稿アカウントのパスワード
    */
    function password()
    {
        return $this->password;
    }

   /**
    * ブログID
    *
    * @return integer ブログID
    */
    function blogId()
    {
        return $this->blogId;
    }

   /**
    * 投稿アカウントのユーザ名を設定する
    *
    * @param string $username 投稿アカウントのユーザ名
    * @return void
    */
    function setUsername($username)
    {
        $this->username = $username;
    }

   /**
    * 投稿アカウントのパスワードを設定する
    *
    * @param string $password 投稿アカウントのパスワード
    * @return void
    */
    function setPassword($password)
    {
        $this->password = $password;
    }

   /**
    * リクエスト結果の文字コードを設定する
    *
    * @param string $output_encode リクエスト結果の文字コード
    * @return void
    */
    function setOutputEncode($output_encode)
    {
        if (WordPress_Encode::isValidEncode($output_encode)) {
            $this->outputEncode = $output_encode;
        } else {
            $this->outputEncode = WordPress_Encode::Utf8;
        }
    }

   /**
    * ブログIDを設定する
    *
    * @param integer $blog_id ブログID
    * @return void
    */
    function setBlogId($blog_id)
    {
        $this->blogId = $blog_id;
    }

   /**
    * ブログURLの正規化
    *
    * FIXME http://example.com/// などが対応できていない。
    *
    * @param string $blog_url ブログURL
    * @return string 正規化されたブログURL
    */
    protected function normalizeBlogUrl($blog_url)
    {
        if ($blog_url[strlen($blog_url) - 1] != '/') {
            $blog_url .= '/';
        }
        return $blog_url;
    }

   /**
    * XML-RPC URLの作成
    *
    * @param string $blog_url ブログURL
    * @return string XML-RPC URL
    */
    protected function buildXmlrpcUrl($blog_url)
    {
        return $blog_url . 'xmlrpc.php';
    }

   /**
    * ブログURLの正当性チェック
    *
    * @param string $blog_url ブログURL
    * @return bool true:正当,false:不正
    */
    protected function isValidBlogUrl($blog_url)
    {
        if (!is_string($blog_url)) return false;
        if (!preg_match('/(http|https):\/\//', $blog_url)) return false;
        return true;
    }
}
